
;byte-sized branch vector example - DednDave

;----------------------------------------------------------------------------------------------

;in this simple example, we want to parse an input string

;we have 5 cases, or "classes", for input values:

;class     parsed characters                           desired result

;  0    null terminator               place a null in the output string and terminate parsing
;  1    ASCII numeric characters      place an ASCII "0" character in the output string
;  2    ASCII upper case letters      place an ASCII "A" character in the output string
;  3    ASCII lower case letters      place an ASCII "a" character in the output string
;  4    all other ASCII characters    ignore - no character is placed in the output string

;----------------------------------------------------------------------------------------------

;we will use an array with byte-sized elements to translate the ASCII characters into vectors

;the array only needs to be generated once during program initialization

;----------------------------------------------------------------------------------------------


        include \masm32\include\masm32rt.inc

        .DATA

;---------------------------------------

;example test string

TstStr  db '23456kjhio;h-sASDFDS3456#%$$^2453ffSAER35347fds@%#$SF',0

;---------------------------------------

;class-to-vector translation table build command string

;  ASCII           Class          Table value (4 * Class)

;    0               0                 0
;   1-2Fh            4                10h
; 30h-39h            1                 4
; 3Ah-40h            4                10h
; 41h-5Ah            2                 8
; 5Bh-60h            4                10h
; 61h-7Ah            3                0Ch
; 7Bh-FFh            4                10h

;each byte pair = (count,4*Class)

Build   db 1,0          ;ASCII 0       = Class 0
        db 2Fh,10h      ;ASCII 1-2Fh   = Class 4
        db 0Ah,4        ;ASCII 30h-39h = Class 1
        db 7,10h        ;ASCII 3Ah-40h = Class 4
        db 1Ah,8        ;ASCII 41h-5Ah = Class 2
        db 6,10h        ;ASCII 5Bh-60h = Class 4
        db 1Ah,0Ch      ;ASCII 61h-7Ah = Class 3
        db 85h,10h      ;ASCII 7Bh-FFh = Class 4
        db 0            ;count 0 = terminate build

;---------------------------------------

        .DATA?

;---------------------------------------

;byte-size class-to-vector translation table

XlatTbl db 256 dup(?)

;---------------------------------------

;output string buffer

OutBuf  db 64 dup(?)

;----------------------------------------------------------------------------------------------

        .CODE

_main   PROC

;---------------------------------------

;build the translation table - this code is executed one time during program initialization

;  ASCII           Class          Table value (4 * Class)

;    0               0                 0
;   1-2Fh            4                10h
; 30h-39h            1                 4
; 3Ah-40h            4                10h
; 41h-5Ah            2                 8
; 5Bh-60h            4                10h
; 61h-7Ah            3                0Ch
; 7Bh-FFh            4                10h

        cld
        mov     esi,offset Build
        xor     eax,eax
        mov     edi,offset XlatTbl
        mov     ecx,eax
        jmp short Build_Loop_Entry

Build_Loop:
        xchg    eax,ecx
        lodsb                  ;get the value
        rep     stosb

Build_Loop_Entry:
        lodsb                  ;get the count
        or      al,al
        jnz     Build_Loop

;---------------------------------------

;display the un-parsed test string

        print   offset TstStr
        print   chr$(13,10,13,10)

;---------------------------------------

;parse the string

        mov     esi,offset TstStr
        mov     edi,offset OutBuf
        jmp short Parse_Loop_Entry

Parse_Loop:
        mov     [edi],al
        inc     edi

Parse_Loop_Entry:
Class4:
        movzx   eax,byte ptr [esi]
        inc     esi
        mov     al,XlatTbl[eax]
        jmp     VectTbl[eax]

Class1: mov     al,'0'
        jmp     Parse_Loop

Class2: mov     al,'A'
        jmp     Parse_Loop

Class3: mov     al,'a'
        jmp     Parse_Loop

Class0: xor     eax,eax
        stosb

;---------------------------------------

;display the parsed string and terminate

        print   offset OutBuf
        print   chr$(13,10)
        exit

;---------------------------------------

        .DATA

;---------------------------------------

;class branch vector table

        ALIGN   4

VectTbl dd Class0,Class1,Class2,Class3,Class4

;---------------------------------------

        .CODE

_main   ENDP

;----------------------------------------------------------------------------------------------

        END     _main














































